import numpy as np


class Perceptron(object):
    """単純なパーセプトロンの分類器

    Parameters
    ------------
    eta : float
      学習率。0.0~1.0
    n_iter : int
      トレーニングの回数
    random_state : int
      重みを初期化するための乱数のシード

    Attributes
    -----------
    w_ : 1d-array
      トレーニング後の重み
    errors_ : list
      各エポックにおけるエラー(誤分類)の数

    """
    def __init__(self, eta=0.01, n_iter=50, random_state=1):
        self.eta = eta
        self.n_iter = n_iter
        self.random_state = random_state

    def fit(self, X, y):
        """トレーニングデータに適合させる下り

        Parameters
        ----------
        X : [n_samples, n_features]
          n_samplesはサンプルの数、 n_featuresは特徴量の数
          トレーニングデータ。機械学習では説明変数とも呼ばれる。
        y : [n_samples]
          最終的に予測したいデータ。機械学習では目的変数と呼ばれている。

        Returns
        -------
        self : object
         selfはその時点での関数自身のこと。
        """
        rgen = np.random.RandomState(self.random_state)
        self.w_ = rgen.normal(loc=0.0, scale=0.01, size=1 + X.shape[1])
        self.errors_ = []

        for _ in range(self.n_iter):
            errors = 0
            for xi, target in zip(X, y): # 各サンプルごとに重みを更新する
                update = self.eta * (target - self.predict(xi))
                self.w_[1:] += update * xi
                self.w_[0] += update
                errors += int(update != 0.0)
            self.errors_.append(errors) # 毎回のエラーを保存しておく
        return self

    def net_input(self, X):
        """入力の総量を計算"""
        return np.dot(X, self.w_[1:]) + self.w_[0]

    def predict(self, X):
        """予測"""
        return np.where(self.net_input(X) >= 0.0, 1, -1)